<?php

namespace FiloBlu\ExtTargetRule\Controller\Products;

use Magento\Catalog\Model\Product as CatalogProduct;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\Result\JsonFactory;

class UpsellAjax extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $_pageFactory;

    /**
     * @var ResultFactory
     */
    protected $_resultFactory;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $_productRepository;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $_coreRegistry;

    /**
     * @var \FiloBlu\ExtTargetRule\Plugin\TargetRuleExtensionPlugin
     */
    protected $_targetRuleExtensionPlugin;

    /**
     * @var \Magento\TargetRule\Model\ResourceModel\Index
     */
    protected $_targetRuleIndex;

    /**
     * UpsellAjax constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $pageFactory
     * @param ResultFactory $resultFactory
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $pageFactory,
        \Magento\Framework\Controller\ResultFactory $resultFactory,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Registry $coreRegistry,
        \FiloBlu\ExtTargetRule\Plugin\TargetRuleExtensionPlugin $targetRuleExtensionPlugin,
        \Magento\TargetRule\Model\Index $targetRuleIndex,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
    )
    {
        $this->_pageFactory = $pageFactory;
        $this->_resultFactory = $resultFactory;
        $this->_productRepository = $productRepository;
        $this->_storeManager = $storeManager;
        $this->_coreRegistry = $coreRegistry;
        $this->_targetRuleExtensionPlugin = $targetRuleExtensionPlugin;
        $this->_targetRuleIndex = $targetRuleIndex;
        $this->_resultJsonFactory = $resultJsonFactory;
        parent::__construct($context);

    }

    public function execute()
    {


        $productId = (int) $this->getRequest()->getParam('id');
        $product = $this->loadProduct($productId);

        $this->_targetRuleIndex->setProduct($product);
        $this->_targetRuleIndex->setType(\Magento\TargetRule\Model\Rule::UP_SELLS);
        $products = $this->_targetRuleExtensionPlugin->getProductIds($this->_targetRuleIndex);

        $data = [];

        $result = $this->_resultJsonFactory->create();
        $resultPage = $this->_pageFactory->create();


        $block = $resultPage->getLayout()
            ->createBlock('\FiloBlu\ExtTargetRule\Block\Catalog\Product\ProductList\Upsell')
            ->setTemplate('FiloBlu_ExtTargetRule::product/list/items.phtml')
            ->setType('upsell-rule')
            ->toHtml();

        $result->setData(['output' => $block]);

        return $result;
    }

    /**
     * Load product model with data by passed id.
     * Return false if product was not loaded or has incorrect status.
     *
     * @param int $productId
     * @return bool|CatalogProduct
     */
    protected function loadProduct($productId)
    {
        if (!$productId) {
            return false;
        }

        try {
            $product = $this->_productRepository->getById($productId);

            if (!in_array($this->_storeManager->getStore()->getWebsiteId(), $product->getWebsiteIds())) {
                throw new NoSuchEntityException();
            }

            if (!$product->isVisibleInCatalog() || !$product->isVisibleInSiteVisibility()) {
                throw new NoSuchEntityException();
            }
        } catch (NoSuchEntityException $noEntityException) {
            return false;
        }

        $this->_coreRegistry->register('current_product', $product);
        $this->_coreRegistry->register('product', $product);

        return $product;
    }
}